@@ -18,7 +18,7 @@ class Context:
 class Message(Context):
     def __init__(self, data: bytes):
         msg = json.loads(data.decode('utf-8'))
-        self.id = msg.id
+        self.id = msg['id']
         self.data = data
 
     def get_id(self) -> str:
