@@ -2,7 +2,6 @@ import inspect, json, sys
 from robomotion.node import Node
 from robomotion.variable import Variable, InVariable, OptVariable, OutVariable, Credentials, ECategory, _Enum
 
-
 class NodeSpec:
     def __init__(self, id: str, icon: str='', name: str='', color: str='', editor: str=None, inputs: int=0, outputs: int=0):
         self.id = id
@@ -92,7 +91,8 @@ class Spec:
                     inProperties, formData = {}, {}
                     uiOrder = []
 
-                    for input in inputVars:
+                    for _input in inputVars:
+                        input = _input['val']
                         inObject = {
                             'type': 'object',
                             'title': input.title,
@@ -116,19 +116,20 @@ class Spec:
                         if input.pyScope:
                             inObject['pyScope'] = True
 
-                        name = Spec.lower_first_letter(input.name)
+                        name = Spec.lower_first_letter(_input['key'])
                         inProperties[name] = inObject
                         pUISchema[name] = {'ui:field': 'variable'}
-                        formData[name] = input.default
+                        formData[name] = {'scope': input.scope, 'name': input.name}
                         uiOrder.append(name)
 
-                    for input in inputs:
+                    for _input in inputs:
+                        input = _input['val']
                         inObject = {
                             'type': input.type,
                             'title': input.title
                         }
 
-                        name = Spec.lower_first_letter(input.name)
+                        name = Spec.lower_first_letter(_input['key'])
                         if input.hidden:
                             pUISchema[name] = {'ui:widget': 'hidden'}
 
@@ -158,7 +159,8 @@ class Spec:
                     outProperties, formData = {}, {}
                     uiOrder = []
 
-                    for output in outputVars:
+                    for _output in outputVars:
+                        output = _output['val']
                         outObject = {
                             'type': 'object',
                             'title': output.title,
@@ -178,19 +180,20 @@ class Spec:
                         if output.messageOnly:
                             outObject['messageOnly'] = True
 
-                        name = Spec.lower_first_letter(output.name)
+                        name = Spec.lower_first_letter(_output['key'])
                         outProperties[name] = outObject
                         pUISchema[name] = {'ui:field': 'variable'}
-                        formData[name] = output.default
+                        formData[name] = {'scope': output.scope, 'name': output.name}
                         uiOrder.append(name)
 
-                    for output in outputs:
+                    for _output in outputs:
+                        output = _output['val']
                         outObject = {
                             'type': output.type,
                             'title': output.title
                         }
 
-                        name = Spec.lower_first_letter(output.name)
+                        name = Spec.lower_first_letter(_output['key'])
                         if output.hidden:
                             pUISchema[name] = {'ui:widget': 'hidden'}
 
@@ -220,7 +223,8 @@ class Spec:
                     optProperties, formData = {}, {}
                     uiOrder = []
 
-                    for option in optionVars:
+                    for _option in optionVars:
+                        option = _option['val']
                         optObject = {
                             'type': 'object',
                             'title': option.title,
@@ -244,13 +248,14 @@ class Spec:
                         if option.pyScope:
                             optObject['pyScope'] = True
 
-                        name = Spec.lower_first_letter(option.name)
+                        name = Spec.lower_first_letter(_option['key'])
                         optProperties[name] = optObject
                         pUISchema[name] = {'ui:field': 'variable'}
-                        formData[name] = option.default
+                        formData[name] = {'scope': option.scope, 'name': option.name}
                         uiOrder.append(name)
 
-                    for option in options:
+                    for _option in options:
+                        option = _option['val']
                         optObject = {
                             'type': option.type,
                             'title': option.title
@@ -266,7 +271,7 @@ class Spec:
                             optObject['enum'] = enums
                             optObject['enumNames'] = enumNames
 
-                        name = Spec.lower_first_letter(option.name)
+                        name = Spec.lower_first_letter(_option['key'])
                         if isinstance(option, Credentials):
                             optObject['subtitle'] = option.title
                             optObject['properties'] = {
@@ -342,7 +347,7 @@ class Spec:
         for key in fields.keys():
             val = fields[key]
             if isinstance(val, Variable) and not isinstance(val, InVariable) and not isinstance(val, OutVariable) and not isinstance(val, OptVariable) and val.input:
-                inputs.append(val)
+                inputs.append({'key': key, 'val': val})
 
         return inputs
 
@@ -353,7 +358,7 @@ class Spec:
         for key in fields.keys():
             val = fields[key]
             if isinstance(val, InVariable):
-                inputs.append(val)
+                inputs.append({'key': key, 'val': val})
 
         return inputs
 
@@ -364,7 +369,7 @@ class Spec:
         for key in fields.keys():
             val = fields[key]
             if isinstance(val, Variable) and not isinstance(val, InVariable) and not isinstance(val, OutVariable) and not isinstance(val, OptVariable) and val.output:
-                outputs.append(val)
+                outputs.append({'key': key, 'val': val})
 
         return outputs
 
@@ -375,7 +380,7 @@ class Spec:
         for key in fields.keys():
             val = fields[key]
             if isinstance(val, OutVariable):
-                outputs.append(val)
+                outputs.append({'key': key, 'val': val})
 
         return outputs
 
@@ -386,7 +391,7 @@ class Spec:
         for key in fields.keys():
             val = fields[key]
             if (isinstance(val, Variable) or isinstance(val, Credentials))  and not isinstance(val, InVariable) and not isinstance(val, OutVariable) and not isinstance(val, OptVariable) and val.option:
-                options.append(val)
+                options.append({'key': key, 'val': val})
 
         return options
 
@@ -397,6 +402,6 @@ class Spec:
         for key in fields.keys():
             val = fields[key]
             if isinstance(val, OptVariable):
-                options.append(val)
+                options.append({'key': key, 'val': val})
 
         return options
